@@ -1,33 +1,24 @@
 package PDL::IO::BareStore;
-our $VERSION = '0.00_02';
+our $VERSION = '0.00_011';
 $VERSION = eval $VERSION;
 use Carp;
 use PDL qw(pdl);
+# byte => 'C'
+# long => 'l'
+# double => 'd'
+# int => ''
 # XXX TODO !!!
 # XS version
 #   madvise() ?
 #   MAP_POPULATE
 #   Direct transform from packed string to PDL? PDL guru wanted !
 # Meta file?
-my %PDL_TYPE = qw(
-  C* byte
-  c* byte
-  s* short
-  S* ushort
-  l* int
-  q* longlong
-  f* float
-  d* double
-);
-
-my %SizeOf = qw( ~~SIZES~~ );
 
 sub new {
   my ($class, %opt) = @_;
   @opt{qw/x y/} = @{delete $opt{'dims'}};
-  $opt{'blockSize'} = $opt{'x'} * $opt{'y'} * $SizeOf{$opt{'type'}};
+  $opt{'blockSize'} = $opt{'x'} * $opt{'y'};
   open $opt{'_F'}, '<:mmap', $opt{'file'} or croak "Error: $!";
-  binmode($opt{'_F'});
   $opt{'readCount'} ||= 1;    # no one use this for reading zero times, right?
   my $self = \%opt;
   bless $self;
@@ -35,23 +26,20 @@ sub new {
 
 sub nextBlock {
   my ($self, $pdl) = @_;
-  my ($F, $l, $x, $y, $t) = @{$self}{qw/_F blockSize x y type /};
-  return 0 if eof($F);
+  my ($F, $l, $x, $y) = @{$self}{qw/_F blockSize x y/};
   my $s;
   my $read = read($F,$s,$l);
+  $$pdl = pdl(unpack($self->{'type'}, $s)); # XXX set pdl type now?
   return -1 if ! defined $read; # error
-  if ( eof($F) || $read == 0 ) {
+  if ( $read == 0 ) {
     if ( --$self->{'readCount'} ) {
       seek $F, 0, 0;
-    }
-  }
-  if ( $read != $self->{'blockSize'} ) {
+    } else { return 0 }  # end
+  } elsif ( $read < $self->{'blockSize'} ) {
     return -1 if $read % $x != 0;
-    $y = $read / ( $x * $SizeOf{$self->{'type'}} );
+    $y = $read / $x;
   }
-  $$pdl = zeroes((new PDL::Type($PDL_TYPE{$t})), $x, $y);
-  ${$$pdl->get_dataref} = $s;
-#  $$pdl->upd_data;
+  $$pdl->reshape($x, $y);
   return 1;
 }
 
@@ -72,7 +60,7 @@ PDL::IO::BareStore - Simple PDL extension for reading 2 dimensional "Big Data"
 
   my $D = new PDL::IO::BareStore(
     file => 'output.rdb',   # file name
-    dims => [14, 100],      # 14 record, 100 line each time
+    dims => [14 * 1, 100],  # 14 record, sizeof(C) is 1, 100 line each time
                             # Not quite handy, any suggestion?
     type => 'C*',           # How you packed the database
     readCount => 5,         # read file for five times
@@ -98,10 +86,6 @@ to use Relational databases to store my data.
 
 =over 8
 
-=item 0.00_02
-
-Bugfixes and improved performance
-
 =item 0.00_011
 
 Fixed Preq, in order to pass the cpan test
